#include "logworker.h"
#include <iostream>
#include <unistd.h>
#include <sys/stat.h>

LogWorker::LogWorker(QObject *parent) : QObject(parent), conn(NULL)
{
}

LogWorker::~LogWorker()
{
    disconnectBackend();
}

/**
 * @brief CrazyflieWorker::connectBackend
 * Command the worker to connect to the backend
 * will set the private class struct, conn, once done.
 * The connected() signal is emitted once complete.
 */
void LogWorker::connectBackend()
{
    if (conn == NULL) {
        conn = ucart_backendConnect();
        emit (connected());
    } else {
        qInfo() << "Attempted to connect crazyflieworker when already connected!";
    }
}

/**
 * @brief CrazyflieWorker::disconnectBackend
 * Command the worker to disconnect from the backend,
 * sets the conn struct to null and emits the
 * disconnected() signal once complete
 */
void LogWorker::disconnectBackend()
{
    if (conn) {
         ucart_backendDisconnect(conn);
         conn = NULL;
         emit (disconnected());
    }
}

void LogWorker::startLogging(QString filename) {
    std::string logFileName = filename.toStdString();
    std::ifstream logFile;
    logFile.open(logFileName.c_str(), std::fstream::ate);
    //std::streamoff p = 0;
    std::string line;
    std::vector<std::string> tokens;
    std::string temp;
    std::stringstream stream;
    QStringList results;
    QString wantedLine;
    uint8_t lineCnt = 0;
    uint8_t nanCnt = 0;

    // while(true) {
    //     logFile.seekg(p);
    //     while(std::getline(logFile, line)) {
    //         QString result(line.c_str());
    //         qInfo() << result;
    //         if(logFile.tellg() == -1) {
    //             p = p + line.size();
    //         }
    //         else {
    //             p = logFile.tellg();
    //         }
    //     }
    //     logFile.clear();
    // }

    while(true) {
        while(std::getline(logFile, line)) {
            //QString result(line.c_str());
            //qInfo() << result;
            if(logFlag == 0) {
                usleep(1000);
                continue;
            }
            else if(line.at(0) == '#') {

            }
            else if(logFlag == 1) {
                lineCnt++;
                stream.str(line);
                while(getline(stream, temp, '\t')) {
                    tokens.push_back(temp);
                }
                wantedLine.append(tokens.at(0).c_str());
                wantedLine.append(",");
                for(int i = 0; i < 5; i++) {
                    if(logColumns.at(i) == -1) {
                        wantedLine.append("NULL,");
                    }
                    else {
                        wantedLine.append(tokens.at(logColumns.at(i)).c_str());
                        wantedLine.append(",");
                        if(!(tokens.at(logColumns.at(i)).compare("nan"))) {
                            nanCnt++;
                        }
                    }
                }
                if(nanCnt >= 5) {
                    nanCnt = 0;
                    wantedLine = "";
                    tokens.clear();
                    stream.str("");
                    stream.clear();
                    lineCnt--;
                    continue;
                }
                results.append(wantedLine);
                wantedLine = "";
                tokens.clear();
                stream.str("");
                stream.clear();
                nanCnt = 0;
                if(lineCnt > 8) {
                    emit(gotLogData(results, false));
                    lineCnt = 0;
                    results.clear();
                }  
            }
            else if(logFlag == 2) {
                emit(gotLogData(results, true));
                results.clear();
                logFlag = 0;
            }
        }
        if(!logFile.eof()) {
            break;
        }
        logFile.clear();
        usleep(150000);
    }
}